QUIET = "%(message)s"
DEV = "[%(name)s:%(lineno)d] %(funcName)s(): %(message)s"
DEV_WITH_LEVEL = "%(levelname)-8s[%(name)s:%(lineno)d] %(funcName)s(): %(message)s"
PROD = "[%(asctime)s] %(levelname)-8s[%(name)s:%(lineno)d] %(funcName)s(): %(message)s "
